"""
constants
"""

import os
import pathlib
import sys

PROGRAM_NAME = os.path.basename(sys.argv[0])

PERSONAL_DOMAINS = [
    "fawong.com",
    "hlx.tw",
    "waf.hk",
    "xilef.org",
    "xn--i8s3q.xn--j6w193g",
]
DEV_DOMAINS = [
    "faw.gg",
    "gyx.io",
    "waf.sexy",
]
COMMUNITY_DOMAINS = [
    "fastandfungible.com",
    "kirinas.com",
    "seris-choice.com",
    "xn--ij2bx6jt8qgte.com",
    "xn--lckwg.net",
]
PUBLIC_DOMAINS = [
    "fantasticrealty.uk",
    "orientelectronic.net",
    "waf.gg",
]
PROJECT_DOMAINS = [
    "mylists.cc",
    "mymovielist.org",
    "mytvlist.org",
]
ADMIN_DOMAINS = [
    "aatf.us",
]
ALL_DOMAINS = (
    PERSONAL_DOMAINS
    + DEV_DOMAINS
    + COMMUNITY_DOMAINS
    + PUBLIC_DOMAINS
    + ADMIN_DOMAINS
    + PROJECT_DOMAINS
)

IP_CIDRS = []

HOME_DIR = os.environ.get("HOME", "/home/waf")

VERSION = (
    open(pathlib.Path(pathlib.Path(__file__).parent, "VERSION"), "r").read().strip()
)
USER_AGENT = f"waflibs python v{VERSION}"



"""
shell utils
"""

import subprocess

from waflibs import log

logger = log.logger().get_logger()


def run_command(
    cmd,
    cwd=None,
    dry_run=False,
    shell=False,
    check=False,
    capture_output=True,
    pipe=False,
    input=None,
    split=True,
    env=None,
):
    """execute shell command"""

    logger.debug(f"original shell command: {cmd}")
    command = cmd
    if split and type(cmd) is str:
        logger.debug("command is of type string... splitting")

        command = cmd.split(" ")
    logger.debug(f"shell command arr: {command}")
    logger.debug(f"shell command to execute: {' '.join(command)}")

    logger.debug(f"env: {env}")
    logger.debug(f"input: {input}")

    if dry_run:
        if capture_output:
            return (
                "would return stdout",
                "would return stderr",
                "would return process",
            )
        else:
            return "would return process"
    else:
        stdout = None
        stderr = None
        if pipe:
            stdout = subprocess.PIPE
            capture_output = False

        process = subprocess.run(
            command,
            text=True,
            cwd=cwd,
            shell=shell,
            capture_output=capture_output,
            check=check,
            input=input,
            stdout=stdout,
            stderr=stderr,
            env=env,
        )
        if capture_output:
            return (process.stdout.strip(), process.stderr.strip(), process)
        else:
            return process


shell_command = run_command
command = run_command
cmd = run_command



#!/usr/bin/env python

import os
import pathlib
import tempfile
import unittest

from waflibs import filedir


class Testfiledir(unittest.TestCase):
    def test_dest_location_dotfile_single_path(self):
        dest = filedir.get_dest_location("fake")

        self.assertEqual(pathlib.Path(f"/tmp/.fake"), dest)

    def test_dest_location_dotfile_path(self):
        dest = filedir.get_dest_location("something/real/fake")

        self.assertEqual(pathlib.Path(f"/tmp/.something/real/fake"), dest)

    def test_dest_location_single_path(self):
        filename = "nothomefake"
        dest = filedir.get_dest_location(filename, dotfile=False)

        self.assertEqual(pathlib.Path(f"/tmp/{filename}"), dest)

    def test_dest_location_target_dir(self):
        dest = filedir.get_dest_location(
            "nothomefake", target_dir="/something/else", dotfile=False
        )

        self.assertEqual(pathlib.Path("/something/else/nothomefake"), dest)

    def test_dest_location_target_dir_dotfile_single(self):
        dest = filedir.get_dest_location("nothomefake", target_dir="/something/else")

        self.assertEqual(pathlib.Path("/something/else/.nothomefake"), dest)

    def test_dest_location_target_dir_dotfile_multiple(self):
        filename = "not/omefake"
        target_dir = "/something/else"
        dest = filedir.get_dest_location(filename, target_dir=target_dir)

        self.assertEqual(pathlib.Path(f"{target_dir}/.{filename}"), dest)

    def test_dest_location_path(self):
        dest = filedir.get_dest_location("rel/sub/not/omefake", dotfile=False)

        self.assertEqual(pathlib.Path(f"/tmp/rel/sub/not/omefake"), dest)



"""
utils for logging
"""

import logging
import os
import random
import sys
import time

import loguru

from waflibs.constants import PROGRAM_NAME

DEBUG = logging.DEBUG
VERBOSE = DEBUG

INFO = logging.INFO


class logger:
    def get_logger(self, name=PROGRAM_NAME):
        """get current logger"""

        return logging.getLogger(name)

    def is_verbose(self, args):
        """check whether verbosity is set"""

        return "verbose" in args and args.verbose

    has_verbose = is_verbose

    def set_all_logger_levels(self, level):
        self.set_level(level)

        loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
        loggers.append(logging.getLogger())
        for logger in loggers:
            logger.setLevel(level)

    def set_level(self, level):
        self.logger.setLevel(level)

    def __init__(
        self,
        args={},
        verbose=False,
        program_name=PROGRAM_NAME,
        log_format=None,
    ):
        """create logger"""

        self.logger = self.get_logger()

        divider = "#" * 55

        if log_format is None:
            log_format = f"""{divider}
timestamp: %(asctime)s
name: %(name)s
filename: %(filename)s
path name: %(pathname)s
module: %(module)s
function name: %(funcName)s()
line num: %(lineno)s
level name: %(levelname)s
level num: %(levelno)s

START LOG
%(message)s
END LOG
{divider}"""
        elif log_format == "simple":
            log_format = None
        formatter = logging.Formatter(log_format)

        if self.is_verbose(args) or verbose:
            self.set_all_logger_levels(VERBOSE)

            sh = logging.StreamHandler()
            sh.setFormatter(formatter)
            self.logger.addHandler(sh)

            fh = logging.FileHandler(
                f"/tmp/{program_name}.{time.time()}.log", mode="a+"
            )
            fh.setFormatter(formatter)
            self.logger.addHandler(fh)
        else:
            self.set_all_logger_levels(INFO)



"""
environment utils
"""

import json
import os

from waflibs import log

logger = log.logger().get_logger()


def get_env_var(var, default=None):
    env_var = os.environ.get(var, default)
    logger.debug(f"env var: {env_var}")

    return env_var


get_env = get_env_var



"""
json utils
"""

from waflibs import log

logger = log.logger().get_logger()


def write_json_file(contents, filename):
    """write json file to disk"""

    if type(filename) == str:
        f = open(filename, "w")
    else:
        f = filename
    f.write(json.dumps(contents, indent=2))
    f.close()



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

import unittest

from waflibs import text

TIMES = 55


class TestText(unittest.TestCase):
    pass


class TestDivider(TestText):
    def test_divider_default(self):
        self.assertEqual(text.divider(), "=" * TIMES)

    def test_divider_char(self):
        self.assertEqual(text.divider(char="-"), "-" * TIMES)

    def test_divider_times(self):
        times = 8

        self.assertEqual(text.divider(times=times), "=" * times)

    def test_divider_all(self):
        times = 9

        self.assertEqual(text.divider(char="_", times=times), "_" * times)


if __name__ == "__main__":
    unittest.main()



"""
uri utils
"""

import ipaddress
import urllib

from waflibs import dns, log

logger = log.logger().get_logger()


def uri_result(result, full_uri=False):
    if full_uri:
        return result
    else:
        return dns.get_domain(result)


def convert_uri(uri, full_uri=False):
    try:
        ipaddress.ip_address(uri)

        result = uri
    except ValueError as e:
        logger.debug(f"exception caught - not an ip address: {e}")

        parse_result = urllib.parse.urlparse(uri)
        logger.debug(f"parse result: {parse_result}")

        if parse_result.netloc:
            result = uri_result(parse_result.hostname, full_uri=full_uri)
        elif parse_result.scheme and not parse_result.netloc:
            result = uri_result(parse_result.scheme, full_uri=full_uri)
        else:
            result = uri_result(parse_result.path, full_uri=full_uri)

    return result



#!/usr/bin/env python
"""bind related dns methods"""

import datetime
import pathlib
import secrets

from waflibs import domain, log, shell

default_bind_dir = "/etc/bind"
default_serial_file = f"{default_bind_dir}/serial"

INFOGREEN = "\033[32m"
INFORED = "\033[31m"
ENDCOLOR = "\033[0m"
BANNER = """\
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;         THIS FILE IS AUTOMATICALLY GENERATED. DO NOT EDIT.
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
"""

logger = log.logger().get_logger()


def restart_bind(sudo=False):
    command_arr = ["service", "bind9", "reload"]
    if sudo:
        command_arr.insert(0, "sudo")
    logger.debug("command to run: {}".format(" ".join(command_arr)))

    stdout, stderr, _ = shell.command(command_arr)
    logger.debug("stdout: {}".format(stdout))
    logger.debug("stderr: {}".format(stderr))


restart = restart_bind


def sign_zone(zone, bind_dir=default_bind_dir, sudo=False):
    salt = secrets.token_hex(8)

    domain = zone
    if zone.endswith("home"):
        domain = ".".join(zone.split(".")[0:-1])

    command_arr = [
        "dnssec-signzone",
        "-S",
        "-3",
        salt,
        "-o",
        domain,
        "-N",
        "INCREMENT",
        "-f",
        f"signed/db.{zone}",
        f"db.{zone}",
    ]
    if sudo:
        command_arr.insert(0, "sudo")
    logger.debug("command to run: {}".format(" ".join(command_arr)))

    stdout, stderr, _ = shell.command(command_arr, cwd=bind_dir)
    logger.debug("stdout: {}".format(stdout))
    logger.debug("stderr: {}".format(stderr))


def sign_zones(
    zones,
    bind_dir=default_bind_dir,
    sudo=False,
):
    for zone in zones:
        sign_zone(zone, bind_dir=bind_dir, sudo=sudo)


def sign_zones_and_restart(
    zones,
    bind_dir=default_bind_dir,
    sudo=False,
):
    sign_zones(zones, bind_dir=bind_dir, sudo=sudo)
    restart_bind(sudo=sudo)


sign_and_restart = sign_zones_and_restart


def generate_serial_number(
    serial_file=default_serial_file,
    serial_number=None,
):
    if serial_number is None:
        now = datetime.datetime.now()
        serial_format = "%Y%m%d"
        date_as_str = now.strftime(serial_format)
        default_serial_number = date_as_str + "00"

        try:
            s_num = open(serial_file).read().strip()
            if s_num.startswith(date_as_str):
                serial_number = int(s_num) + 1
            else:
                serial_number = default_serial_number
        except FileNotFoundError as e:
            logger.debug("serial file not found... ignoring")
            logger.debug(f"error: {e}")

            serial_number = default_serial_number

    return int(serial_number)


def write_serial_number(serial_number, serial_file):
    try:
        with open(serial_file, "w") as f:
            f.write(f"{serial_number}\n")
    except PermissionError as e:
        logger.debug(f"error writing to {serial_file}: {e}")


write_serial = write_serial_number


gen_serial_number = generate_serial_number
gen_serial_num = generate_serial_number
gen_serial = generate_serial_number
generate_serial = generate_serial_number


def generate_bind_file(
    all_records,
    domain_name,
    zone_file_name=None,
    template_file_name=None,
    domain_template_file_name=None,
    serial_number=None,
    bind_dir=default_bind_dir,
    serial_file=default_serial_file,
    logger=logger,
    dry_run=False,
):
    """generate bind file from given arguments"""

    if not serial_number:
        serial_number = generate_serial(
            serial_file=serial_file,
        )
    write_serial(serial_number, serial_file)

    text.eprint(
        f"{INFOGREEN}Generating zone records for {domain}...{ENDCOLOR} ",
        newline=False,
    )

    logger.debug("all records: {}".format(all_records))
    count = 0
    records = ""
    for record in all_records:
        record_name = record["name"]
        record_type = record["type"]
        content = record["content"]
        proxied = record["proxied"]
        count += 1

        logger.debug(
            f"record name: {domain.idna(record_name)}, record type: {record_type}"
        )
        logger.debug(f"record content: {content}, proxied: {proxied}")

        if record_type in ("MX"):
            records += (
                f"{record_name} IN {record_type} {record['priority']} {content}\n"
            )
        elif record_type in ("SRV"):
            records += f"{record_name} IN {record_type} {record['data']['priority']} {content}\n"
        elif record_type in ("SPF", "TXT"):
            records += f'{record_name} IN {record_type} "{content}"\n'
        elif record_type in ("NS", "CNAME"):
            records += f"{record_name} IN {record_type} {content}\n"
        elif record_type in ("A", "AAAA"):
            records += f"{record_name} IN {record_type} {content}\n"
        else:
            logger.fatal(f"unsupported dns record type {record_type}")

    records += f"""\n; {count} total records found"""

    text.eprint(f"{INFOGREEN}done.{ENDCOLOR}")

    if zone_file_name is None:
        zone_file_name = f"db.{domain_name}"
    zone_file = pathlib.Path(bind_dir, zone_file_name)
    logger.debug(f"zone file: {zone_file}")

    if template_file_name:
        template_file = open(template_file_name)
        if domain_template_file_name:
            domain_template_file = open(domain_template_file_name)
            tmpl = template_file.read() + "\n" + domain_template_file.read() + "\n"
            domain_template_file.close()
        else:
            tmpl = template_file.read() + "\n"
        logger.debug(f"template: {tmpl}")

        nameserver = "ns.home.waf.hk"
        contents = tmpl.format(
            banner=BANNER,
            zone=domain_name,
            records=records,
            serial=serial_number,
            nameserver=nameserver,
        )
        template_file.close()
    else:
        contents = f"{BANNER}\n{records}"

    logger.debug(f"bind file: {contents}")

    return write_zone_file(zone_file, contents, dry_run)


def write_zone_file(zone_file, contents, dry_run=False):
    if dry_run:
        logger.info(f"{INFOGREEN}would write dns zone file to {zone_file}{ENDCOLOR}")
        logger.debug(f"contents: {contents}")
    else:
        logger.debug(f"{INFOGREEN}Writing to file {zone_file}...{ENDCOLOR}")
        try:
            with open(zone_file, "w") as f:
                f.write(contents)
                logger.debug(
                    f"{INFOGREEN}finished writing to file {zone_file}{ENDCOLOR}"
                )

            return zone_file
        except IOError as ioe:
            logger.debug(f"{INFORED}ERROR!{ENDCOLOR}")
            logger.fatal(ioe)



#!/usr/bin/env python
"""utils for mysql stuffs"""

import mysql.connector
import psycopg


def mysql_connect(host, user, password, database, buffered=False):
    """mysql connector"""

    return mysql.connector.connect(
        host=host,
        user=user,
        password=password,
        db=database,
        buffered=buffered,
    )


def postgresql_connect(host, user, password, database):
    """postgresql connector"""

    return psycopg.connect(
        dbname=database,
        user=user,
        password=password,
        host=host,
    )


postgres_connect = postgresql_connect



#!/usr/bin/env python
"""errors and exceptions"""


class ValidationError(Exception):
    """generic validation error"""

    def __init__(self, m):
        self.message = m

    def __str__(self):
        return self.message
